@@ -10,7 +10,7 @@ from scipy.stats import trim_mean
 import math
 
 from skimage.filters import threshold_otsu, gaussian, threshold_local
-from skimage.morphology import remove_small_objects
+from skimage.morphology import remove_small_objects, convex_hull_object
 from skimage.segmentation import clear_border
 from skimage.util import invert
 from skimage.measure import regionprops, label
@@ -60,9 +60,9 @@ def make_grid_auto(im, grid):
         med = trim_mean(peaks[1:] - peaks[:-1], 0.2)
         #for bad input images the distance between colonies times the number of rows/columns can exceed image dimensions. 
         #In this case, guess the distance based on image dimensions and number of colonies
-        if med*n > len(imvals):
+        if med*(n-1) > len(imvals):
             print('Could not detect enough peaks. Guessing grid positions. Please check QC images carefully.')
-            med = (len(imvals)-0.2*len(imvals))/n
+            med = (len(imvals)-0.1*len(imvals))/(n-1)
 
         #create hypothetical, ideal grid based on mean distance
         to_fit = np.linspace(0, med*(n-1),n)
@@ -120,7 +120,7 @@ def match_to_grid(labels, centroids, grid, griddist, d=3, reportAll=False):
     return dm
 
 
-def make_mask(image, t=1, s=1, hardImageThreshold=None, hardSizeThreshold=None, local=False):
+def make_mask(image, t=1, s=1, hardImageThreshold=None, hardSizeThreshold=None, local=False, convexhull=False):
     '''
     Identifies suitable morphological components from image by thresholding.
     '''
@@ -136,6 +136,10 @@ def make_mask(image, t=1, s=1, hardImageThreshold=None, hardSizeThreshold=None,
             
         mask = image>thresh
     
+    #Fill holes. Warning: takes a long time
+    if convexhull:
+        mask = convex_hull_object(mask)
+        
     #Filter small components. The default threshold is 0.00005 of the image area 
     if hardSizeThreshold:
         size_thresh = hardSizeThreshold
@@ -143,7 +147,6 @@ def make_mask(image, t=1, s=1, hardImageThreshold=None, hardSizeThreshold=None,
         size_thresh = s * np.prod(image.shape) * 0.00005
     mask = remove_small_objects(mask, min_size=size_thresh)
     
-    #Fill holes?? In future
     
     #Clear border
     mask = clear_border(mask)
@@ -161,8 +164,8 @@ def check_and_negate(orig_image, negate=True):
     
     #Check if images are grayscale and convert if necessary
     if len(image.shape) == 3:
-        warn('The following is not a greyscale image and will be converted before processing: %s'%images.files[i])
-        image = image[:,:,0] + image[:,:,1] + image[:,:,2]
+        warn('Image is not in greyscale, converting before processing')
+        image = image.astype(int).mean(axis=2)
 
     #Convert to float and re-scale to [0,1]            
     image = image.astype(float)
@@ -174,7 +177,7 @@ def check_and_negate(orig_image, negate=True):
         
     return image
 
-def quantify_single_image_size(orig_image, grid, auto, t=1, d=3, s=1, negate=True, reportAll=False, hardImageThreshold=None, hardSizeThreshold=None):
+def quantify_single_image_size(orig_image, grid, auto, t=1, d=3, s=1, negate=True, reportAll=False, hardImageThreshold=None, hardSizeThreshold=None, localThresh=None, convexhull=False):
     '''
     Process a single image to extract colony sizes.
     '''
@@ -189,7 +192,7 @@ def quantify_single_image_size(orig_image, grid, auto, t=1, d=3, s=1, negate=Tru
         grid, griddist = make_grid(grid)
         
     #Make mask
-    mask = make_mask(image, t=t, s=s, hardImageThreshold=hardImageThreshold, hardSizeThreshold=hardSizeThreshold, local=False)
+    mask = make_mask(image, t=t, s=s, hardImageThreshold=hardImageThreshold, hardSizeThreshold=hardSizeThreshold, local=localThresh, convexhull=convexhull)
     
     #Measure regionprobs
     data = {r.label : {p : r[p] for p in ['label', 'area', 'centroid', 'mean_intensity', 'perimeter']} for r in regionprops(mask, intensity_image=image)}
@@ -256,7 +259,7 @@ def quantify_single_image_redness(orig_image, grid, auto, t=1, d=3, s=1, negate=
 
     return (data, qc)
     
-def quantify_batch(images, grid, auto, mode, qc='qc_images', out='pyphe_quant', t=1, d=3, s=1, negate=True, reportAll=False, reportFileNames=None, hardImageThreshold=None, hardSizeThreshold=None):
+def quantify_batch(images, grid, auto, mode, qc='qc_images', out='pyphe_quant', t=1, d=3, s=1, negate=True, reportAll=False, reportFileNames=None, hardImageThreshold=None, hardSizeThreshold=None, localThresh=None, convexhull=False):
     '''
     Analyse colony size for batch of plates. Depending on mode, either the quantify_single_image_grey or quantify_single_image_redness function is applied to all images.
     '''
@@ -264,7 +267,7 @@ def quantify_batch(images, grid, auto, mode, qc='qc_images', out='pyphe_quant',
     for fname, im in zip(images.files, images):
         
         if mode == 'batch':
-            data, qc_image = quantify_single_image_size(np.copy(im), grid, auto, t=t, d=d, s=s, negate=negate, reportAll=reportAll, hardImageThreshold=hardImageThreshold, hardSizeThreshold=hardSizeThreshold)
+            data, qc_image = quantify_single_image_size(np.copy(im), grid, auto, t=t, d=d, s=s, negate=negate, reportAll=reportAll, hardImageThreshold=hardImageThreshold, hardSizeThreshold=hardSizeThreshold, localThresh=localThresh, convexhull=convexhull)
         elif mode == 'redness':
             data, qc_image = quantify_single_image_redness(np.copy(im), grid, auto, t=t, d=d, s=s, negate=negate, reportAll=reportAll, hardImageThreshold=hardImageThreshold, hardSizeThreshold=hardSizeThreshold)
         else:
@@ -321,7 +324,7 @@ def quantify_single_image_fromTimecourse(orig_image, mask, negate=True, calibrat
     return data
 
         
-def quantify_timecourse(images, grid, auto, qc='qc_images', out='pyphe_quant', t=1, d=3, s=1, negate=True, reportAll=False, reportFileNames=False, hardImageThreshold=None, hardSizeThreshold=None, calibrate='x', timepoints=None):
+def quantify_timecourse(images, grid, auto, qc='qc_images', out='pyphe_quant', t=1, d=3, s=1, negate=True, reportAll=False, reportFileNames=False, hardImageThreshold=None, hardSizeThreshold=None, calibrate='x', timepoints=None, localThresh=None, convexhull=False):
     '''
     Analyse a timeseries of images. Make the mask based on the last image and extract intensity information from all previous images based on that.
     '''
@@ -338,7 +341,7 @@ def quantify_timecourse(images, grid, auto, qc='qc_images', out='pyphe_quant', t
         grid, griddist = make_grid(grid)
     
     #Make mask
-    mask = make_mask(fimage, t=t, s=s, hardSizeThreshold=hardSizeThreshold)
+    mask = make_mask(fimage, t=t, s=s, hardSizeThreshold=hardSizeThreshold, convexhull=convexhull)
     
     #Make table of intensities over time
     data = {fname : quantify_single_image_fromTimecourse(orig_image, mask, negate=negate, calibrate=calibrate) for fname,orig_image in zip(images.files, images)}
